#define PROC_CPP
#include "proc.h"
#include <mmsystem.h>
#include "cstl/map.h"
#pragma comment(lib, "winmm.lib")
#ifndef _DEBUG

#define malloc(size)	HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, (size))
#define free(p)		HeapFree(GetProcessHeap(), 0, (p))

#endif

EXTERN_C int _fltused = 0x9875;	// LNK2019(tmainCRTStartup)΍

CSTL_MAP_INTERFACE(IntIntMap, int, int)
CSTL_MAP_IMPLEMENT(IntIntMap, int, int, CSTL_LESS)

// ϐ
static HINSTANCE hNtDll;
static PFN_NTQUERYSYSTEMINFORMATION fnProcInfo;
static DWORD LastCheckTime;
static IntIntMap *LastTimeMap = NULL;
static DWORD numCpu;


// msec
int lLarge2Int( LARGE_INTEGER large )
{
	SYSTEMTIME stTime;
	int iTime;

	FileTimeToSystemTime((FILETIME*) &large, &stTime );

	iTime = stTime.wSecond * 1000;
	iTime += stTime.wMilliseconds;

	return iTime;
}

BOOL IniteNtdll(void) {
	SYSTEM_INFO sysInfo;

	// Load Library
	hNtDll = LoadLibraryW(L"ntdll.dll");
	if (hNtDll == NULL) {
		return FALSE;
	}

	fnProcInfo = PFN_NTQUERYSYSTEMINFORMATION(GetProcAddress(hNtDll, "NtQuerySystemInformation"));

	if (fnProcInfo == NULL) {
		FreeLibrary(hNtDll);
		return FALSE;
	}
	GetSystemInfo(&sysInfo);
	numCpu = sysInfo.dwNumberOfProcessors;

	return TRUE;
}

void CloseNtdll(void) {
	if (hNtDll) {
		FreeLibrary(hNtDll);
	}
	if (LastTimeMap) {
		IntIntMap_delete(LastTimeMap);
	}
}

PSYSTEM_PROCESS_INFORMATION getProcessInfo(void) {
	ULONG ulSize;
	LONG status;
	LPBYTE pBuffer = NULL;
	ULONG bufferSize;

	bufferSize = 0;
	do {
		status = fnProcInfo(SystemProcessInformation, pBuffer, bufferSize, &ulSize);
		if (status == 0xC0000004 /*STATUS_INFO_LENGTH_MISMATCH*/) {
			free(pBuffer);
			bufferSize = ulSize;
			pBuffer = (LPBYTE)malloc(bufferSize);
		}
	} while (status == 0xC0000004 /*STATUS_INFO_LENGTH_MISMATCH*/);

	if(status == 0x00) {
		return (PSYSTEM_PROCESS_INFORMATION)pBuffer;
	}

	return NULL;
}

PSYSTEM_PROCESS_INFORMATION getByPID(PSYSTEM_PROCESS_INFORMATION head, DWORD pid) {
	PSYSTEM_PROCESS_INFORMATION current = head;

	while (current != NULL) {
		if (current->ProcessId == (DWORD)pid) {
			return current;
		}

		if(current->NextEntryOffset == 0) {
			current = NULL;
		} else {
			current = (PSYSTEM_PROCESS_INFORMATION)((LPBYTE)current + current->NextEntryOffset);
		}
	}

	return NULL;
}

BOOL getTopProcess(SIMPLE_PROC_INFO *top_proc) {
	IntIntMap *NewTimeMap;
	PSYSTEM_PROCESS_INFORMATION head;
	PSYSTEM_PROCESS_INFORMATION proc;
	DWORD NewCheckTime;
	DWORD difftime;
	DOUBLE usage;
	DOUBLE topUsage = -1.0;
	DWORD topID = 0;
	DWORD newuser;
	DWORD newkernel;

	NewTimeMap = IntIntMap_new();
	NewCheckTime = timeGetTime();
	difftime = NewCheckTime - LastCheckTime;
	head = getProcessInfo();
	if (!head) return FALSE;
	proc = head;
	while(true) {
		
		newuser = lLarge2Int(proc->UserTime);
		newkernel = lLarge2Int(proc->KernelTime);
		IntIntMap_insert(NewTimeMap, proc->ProcessId, newuser + newkernel, NULL);

		if (LastTimeMap && IntIntMap_find(LastTimeMap, proc->ProcessId) != IntIntMap_end(LastTimeMap)) {
			usage = ((100.0*(DOUBLE)(newuser + newkernel - *(IntIntMap_lookup(LastTimeMap, proc->ProcessId))))/(DOUBLE)(difftime))/(DOUBLE)numCpu;
		} else {
			int createtime = lLarge2Int(proc->CreateTime);
			usage = ((100.0*(DOUBLE)(newuser + newkernel))/(DOUBLE)(NewCheckTime - createtime))/(DOUBLE)numCpu;
		}
		if (proc->ProcessId && topUsage < usage) {
			topUsage = usage;
			topID = proc->ProcessId;
		}
		proc = _NextProcess(proc);
		if (proc == NULL) break;
	}
	top_proc->PID = topID;
	top_proc->usage = topUsage;
	proc = getByPID(head, topID);
	if (proc->ImageName.Buffer) {
		lstrcpyW(top_proc->name, proc->ImageName.Buffer);
	} else if (proc->ProcessId == 0) {
		lstrcpyW(top_proc->name, L"System Idle Process");
	}
	free(head);
	LastCheckTime = NewCheckTime;

	IntIntMap_delete(LastTimeMap);
	LastTimeMap = NewTimeMap;
	return TRUE;
}